unit uVumeter;

interface

uses
  SysUtils,
  VrLevelBar;

type
  TVuMeter = Record
    Vu: TVrLevelBar;
    Name: string;
    MinValue: double;
    MaxValue: double;
    HalfLife: double; // time in sec to decay to half value of last peak
    Scalar: double;
    Output: double;
  end;
  TVumeters = record
    Left: TVuMeter;
    Right: TVuMeter;
    T0: cardinal;
    T1: cardinal;
    MinLevelInDb: double;
    MaxLevelInDb: double;
    RefreshInterval: integer; // in ms
    ResetInProgress: boolean;
  end;

var
  FVumeters: TVumeters;

procedure Vumeters_Init;
procedure VuMeters_Reset;
procedure VuMeters_SetLimits(iMin, iMax: integer);
procedure VuMeter_SetLimits(vu: TVuMeter; sPrefix: string; iMin, iMax: integer);
procedure VuMeters_SetMinValue(iVal: integer);
procedure VuMeter_SetMinValue(vu: TVuMeter; iVal: integer);
procedure VuMeters_SetMaxValue(iVal: integer);
procedure VuMeter_SetMaxValue(vu: TVuMeter; iVal: integer);
procedure VuMeter_SetPosition(vu: TVuMeter; iVal: double);

implementation

uses
  uRes,
  frmMain;

procedure Vumeters_Init;
begin
  with FVumeters do
  begin
    Left.vu := fmMain.VuAudioLeft;
    Left.Name := sLeft;
    Right.vu := fmMain.VuAudioRight;
    Right.Name := sRight;
  end;
end;

procedure Vumeters_Reset;
begin
  with FVumeters do
  begin
    ResetInProgress := true;
    Left.Output := FAudioParams.MinLevelInDb;
    Right.Output := FAudioParams.MinLevelInDb;
    VuMeter_SetPosition(Left, FAudioParams.MinLevelInDb);
    VuMeter_SetPosition(Right, FAudioParams.MinLevelInDb);
    ResetInProgress := false;
  end;
end;

procedure VuMeters_SetLimits(iMin, iMax: integer);
begin
  with FVumeters do
  begin
    VuMeter_SetLimits(Left, sLeftLevel, iMin, iMax);
    VuMeter_SetLimits(Right, sRightLevel, iMin, iMax);  end;
end;

procedure VuMeter_SetLimits(vu: TVuMeter; sPrefix: string; iMin, iMax: integer);
begin
  if not (Vu.Vu is TvrLevelBar) then exit;
  Vu.MinValue := iMin;
  Vu.MaxValue := iMax;
  (Vu.Vu as TvrLevelBar).MinValue := iMin;
  (Vu.Vu as TvrLevelBar).MaxValue := iMax;
  (Vu.Vu as TvrLevelBar).Hint :=
    format('%s%sMax = %d dB %sMin = %d dB', [sPrefix, #13, iMax, #13, iMin]);
end;

procedure VuMeters_SetMinValue(iVal: integer);
begin
  with FVumeters do
  begin
    VuMeter_SetMinValue(Left, iVal);
    VuMeter_SetMinValue(Right, iVal);
  end;
end;

procedure VuMeter_SetMinValue(vu: TVuMeter; iVal: integer);
begin
  if not (Vu.Vu is TvrLevelBar) then exit;
  (Vu.Vu as TvrLevelBar).MinValue := iVal;
end;

procedure VuMeters_SetMaxValue(iVal: integer);
begin
  with FVumeters do
  begin
    VuMeter_SetMaxValue(Left, iVal);
    VuMeter_SetMaxValue(Right, iVal);
  end;
end;

procedure VuMeter_SetMaxValue(vu: TVuMeter; iVal: integer);
begin
  if not (Vu.Vu is TvrLevelBar) then exit;
  (Vu.Vu as TvrLevelBar).MaxValue := iVal;
end;

procedure VuMeter_SetPosition(vu: TVuMeter; iVal: double);
begin
  if not (Vu.Vu is TvrLevelBar) then exit;
  try
    (Vu.Vu as TvrLevelBar).BeginUpdate;
    (Vu.Vu as TvrLevelBar).Position := trunc(iVal);
    (Vu.Vu as TvrLevelBar).EndUpdate;
  except
    (Vu.Vu as TvrLevelBar).BeginUpdate;
    (Vu.Vu as TvrLevelBar).Position := (Vu.Vu as TvrLevelBar).MinValue;
    (Vu.Vu as TvrLevelBar).EndUpdate;
  end;
end;


end.
